import {ComponentType} from 'react';
import {useState, useEffect} from 'react';
import {Route, Link, generatePath, matchPath, useHistory, useLocation} from 'react-router-dom';

const routes = {

}

export const NamedRoute = ({name, routeName, query, pendingQueryComponent:P, errorQueryComponent:E, errorQueryComponentErrorProp="error", componentQueryResultProp='data', path, component, render, ...props}) => {
    name  ||= routeName
    routes[name || path] = {...props, path}

    const Render:ComponentType = component || render 

    const ComponentWrapper = (props) => {
        const [queryResult, setQueryResult] = useState()
        const [queryError, setQueryError] = useState()
        useEffect(() => {
            console.log(props)
            if (query && typeof query === 'function'){
                const querySyncResult = query(props.match.params, props, name)
                if (querySyncResult && typeof querySyncResult.then === 'function'){
                    querySyncResult.then(setQueryResult, setQueryError)
                }
                else {
                    setQueryResult(querySyncResult)
                }
            }
            return () => {
                setQueryError()
                setQueryResult()
            }
        },Object.values(props.match.params))

        if (!query || typeof query !== 'function')
            return <Render {...{[componentQueryResultProp]:query, ...props}} />

        if (!queryResult && !queryError)
            return P ? <P {...props} /> : null

        if (queryError)
            return E ? <E {...{[errorQueryComponentErrorProp]: queryError, ...props}} /> : null

        return <Render {...{[componentQueryResultProp]:queryResult, ...props}} />
    }

    return <Route {...props} path={path}  component={ComponentWrapper}/>
}

export const NamedLink = ({routeName, params, to,...props}) => {
    const route = routes[routeName]
    if (route && !to){
        const pattern = route.path
        to            = generatePath(pattern, params)
    }
    return <Link to={to} {...props} />
}

export const HashRoute = ({component, render, ...props}) => {
    const history = useHistory()
    const location = useLocation()
    const [match, setMatch] = useState<null|object>(null)
    const checkMatch = () => {
        setMatch(matchPath(window.location.hash.slice(1), props))
    }
    useEffect(checkMatch,[props.path, props.exact, props.strict, props.sensitive])
    useEffect(() => {
        window.addEventListener('hashchange', checkMatch)
        return () => {
            window.removeEventListener('hashchange', checkMatch)
        }
    },[])
    if (!match) return null

    const Render:ComponentType = component || render 

    return <Render match={match} history={history} location={location} />
}

type GetUserRolesFunc = () => Array<string>

export const createPrivateRoute = (getUserRoles:GetUserRolesFunc, defaultFallback:string|Function, defaultRouteRoles:string[]):Function => 
    (RouteComponent:ComponentType):ComponentType => 
        ({fallback, roles=[], render, component, ...props}) => {
            const Render:ComponentType = component || render 

            const intersectArrays = (first:Array<any>, second:Array<any>) => 
                first.find(firstItem => second.find(secondItem => firstItem === secondItem))

            const ComponentWrapper:ComponentType = (props) => {
                fallback               ||= defaultFallback
                roles:Array<string>    ||= defaultRouteRoles

                const userRoles:Array<string> = getUserRoles()

                if (!fallback){
                    throw new ReferenceError('fallback or defaultFallback are mandatory for Private Route')
                }

                if (!intersectArrays(roles, userRoles)){
                    if (typeof fallback === 'string')
                        props.history.push(fallback)
                    if (typeof fallback === 'function')
                        fallback(props.history, props)
                    return null;
                }


                return <Render {...props} />
            }

            if (!getUserRoles || typeof getUserRoles !== 'function'){
                throw new ReferenceError('getUserRoles are mandatory for Private Routing and it should be a function')
            }

            return <RouteComponent {...props} component={ComponentWrapper}/>
        }

//console.log(createPrivateRoute)

    

//export const HashNamedRoute = ({name, routeName, query, pendingQueryComponent:P, errorQueryComponent:E, errorQueryComponentErrorProp="error", componentQueryResultProp='data', path, component, render, ...props}) => {


